/*****************************************************************************************************************************
--
--    File Name    : AXI_master_read.v

--    Description  : This module implements AXI master read controller for read channels 2, 3 and 3
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

********************************************************************************************************************************/
`timescale 1ns/1ps
module AXI_master_read #(parameter g_AXI_AWIDTH = 32,
                         parameter g_AXI_DWIDTH = 64,
                         parameter g_AXI_BUFF_AWIDTH = 12)
                       (input wire clock,
                        input wire nreset,
                        input wire[(g_AXI_AWIDTH - 1) : 0] axi_rd_ddr_addr,
                        input wire start_axi_tr,
                        input wire[4 : 0] burst_length,
                        input wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read,
                        output reg transf_compl,
                        output reg master_rd_req,
                        input wire arbitr_ack,
                        output wire[(g_AXI_AWIDTH - 1) : 0] axi_saddr,
                        output wire[2 : 0] axi_tr_size,
                        output wire[1 : 0] axi_burst_typ,
                        output reg[3 : 0] axi_burst_len,
                        output wire[1 : 0] axi_tr_dir,
                        output reg axi_start,
                        input wire axi_tr_done,
                        input wire[(g_AXI_DWIDTH - 1) : 0] axi_data_in,
                        input wire axi_if_wen,
                        output wire[(g_AXI_DWIDTH - 1) : 0] axi_buff_data_o,
                        output wire[(g_AXI_BUFF_AWIDTH - 1) : 0] axi_buff_addr,
                        output wire axi_buff_wen);
  reg[(g_AXI_AWIDTH - 1) : 0] li;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] oi;
  parameter[3 : 0] ii = 4'b1111;
  parameter[3 : 0] OOI = 4'b1110;
  parameter[3 : 0] IOI = 4'b1101;
  parameter[3 : 0] lOI = 4'b1100;
  parameter[3 : 0] oOI = 4'b1011;
  parameter[3 : 0] iOI = 4'b1010;
  parameter[3 : 0] OII = 4'b1001;
  parameter[3 : 0] III = 4'b1000;
  parameter[3 : 0] lII = 4'b0111;
  parameter[3 : 0] oII = 4'b0110;
  parameter[3 : 0] iII = 4'b0101;
  parameter[3 : 0] OlI = 4'b0100;
  parameter[3 : 0] IlI = 4'b0011;
  parameter[3 : 0] llI = 4'b0010;
  parameter[3 : 0] olI = 4'b0001;
  parameter[3 : 0] ilI = 4'b0000;
  parameter[2 : 0] O0I = 3'b011;
  parameter[1 : 0] I0I = 2'b00;
  parameter[1 : 0] l0I = 2'b01;
  parameter[2 : 0] o0I = 3'd0,
                   i0I = 3'd1,
                   O1I = 3'd2,
                   I1I = 3'd3,
                   l1I = 3'd4,
                   o1I = 3'd5;
  reg[2 : 0] i1I;
  reg OoI;
  reg[4 : 0] IoI;
  assign axi_burst_typ = l0I;
  assign axi_saddr = li;
  assign axi_tr_size = O0I;
  assign axi_tr_dir = I0I;
  always
    @(posedge clock or
      negedge nreset)
    begin
      if (!nreset)
        begin
          i1I <= o0I;
        end
      else
        begin
          case (i1I)
            o0I:
              begin
                if (start_axi_tr)
                  begin
                    i1I <= i0I;
                  end
                else
                  begin
                    i1I <= o0I;
                  end
              end
            i0I:
              begin
                i1I <= O1I;
              end
            O1I:
              begin
                i1I <= I1I;
              end
            I1I:
              begin
                if (arbitr_ack)
                  begin
                    i1I <= l1I;
                  end
                else
                  begin
                    i1I <= I1I;
                  end
              end
            l1I:
              begin
                if (axi_tr_done)
                  begin
                    i1I <= o1I;
                  end
                else
                  begin
                    i1I <= l1I;
                  end
              end
            o1I:
              begin
                i1I <= o0I;
              end
            default:
              i1I <= o0I;
          endcase
        end
    end
  always
    @(posedge clock or
      negedge nreset)
    begin
      if (!nreset)
        begin
          li <= { g_AXI_AWIDTH { 1'b0 } };
          axi_start <= 1'b0;
          master_rd_req <= 1'b0;
          transf_compl <= 1'b0;
          axi_burst_len <= ii;
          OoI <= 1'b0;
          IoI <= 5'd0;
        end
      else
        begin
          case (i1I)
            o0I:
              begin
                axi_start <= 1'b0;
                master_rd_req <= 1'b0;
                transf_compl <= 1'b0;
                axi_burst_len <= ii;
                OoI <= 1'b0;
                if (start_axi_tr)
                  begin
                    li <= axi_rd_ddr_addr;
                    IoI <= burst_length;
                  end
                else
                  begin
                    li <= { g_AXI_AWIDTH { 1'b0 } };
                    IoI <= 5'd0;
                  end
              end
            i0I:
              begin
                case (IoI)
                  5'd1:
                    begin
                      axi_burst_len <= ilI;
                    end
                  5'd2:
                    begin
                      axi_burst_len <= olI;
                    end
                  5'd3:
                    begin
                      axi_burst_len <= llI;
                    end
                  5'd4:
                    begin
                      axi_burst_len <= IlI;
                    end
                  5'd5:
                    begin
                      axi_burst_len <= OlI;
                    end
                  5'd6:
                    begin
                      axi_burst_len <= iII;
                    end
                  5'd7:
                    begin
                      axi_burst_len <= oII;
                    end
                  5'd8:
                    begin
                      axi_burst_len <= lII;
                    end
                  5'd9:
                    begin
                      axi_burst_len <= III;
                    end
                  5'd10:
                    begin
                      axi_burst_len <= OII;
                    end
                  5'd11:
                    begin
                      axi_burst_len <= iOI;
                    end
                  5'd12:
                    begin
                      axi_burst_len <= oOI;
                    end
                  5'd13:
                    begin
                      axi_burst_len <= lOI;
                    end
                  5'd14:
                    begin
                      axi_burst_len <= IOI;
                    end
                  5'd15:
                    begin
                      axi_burst_len <= OOI;
                    end
                  5'd16:
                    begin
                      axi_burst_len <= ii;
                    end
                  default:
                    begin
                      axi_burst_len <= ii;
                    end
                endcase
              end
            O1I:
              begin
                master_rd_req <= 1'b1;
                axi_start <= 1'b1;
              end
            I1I:
              begin
                if (arbitr_ack)
                  begin
                    axi_start <= 1'b0;
                  end
                else
                  begin
                    axi_start <= axi_start;
                  end
              end
            l1I:
              begin
                if (axi_tr_done)
                  begin
                    master_rd_req <= 1'b0;
                  end
                else
                  begin
                    master_rd_req <= master_rd_req;
                  end
              end
            o1I:
              begin
                transf_compl <= 1'b1;
                OoI <= 1'b1;
              end
            default:
              begin
              end
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        oi <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
      else
        begin
          if ((oi == locs_to_read) && (OoI))
            oi <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          else
            if (axi_if_wen)
              oi <= oi + 'd1;
        end
    end
  assign axi_buff_data_o = axi_data_in;
  assign axi_buff_addr = oi;
  assign axi_buff_wen = axi_if_wen;
endmodule
