/*****************************************************************************************************************************
--
--    File Name    : data_unpack_64_32_image.v 

--    Description  : This module implements 64-bit to 32-bit unpacker for read channels 2, 3 and 4
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module unpack_64_32 #(parameter g_AXI_DWIDTH = 64,
                      parameter g_AXI_BUFF_AWIDTH = 10,
                      parameter g_RD_CHANNEL_VIDEO_DATA_WIDTH = 32)
                    (input wire clock,
                     input wire nreset,
                     input wire start_unpack,
                     input wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read,
                     input wire[(g_AXI_DWIDTH - 1) : 0] buff_data_in,
                     output reg[(g_AXI_BUFF_AWIDTH - 1) : 0] buff_raddr,
                     output reg[(g_RD_CHANNEL_VIDEO_DATA_WIDTH - 1) : 0] video_data,
                     output reg data_valid,
                     output reg unpacker_compl);
  parameter[2 : 0] lol = 3'd0,
                   ool = 3'd1,
                   iol = 3'd2,
                   Oil = 3'd3,
                   Iil = 3'd4,
                   OI0 = 3'd5;
  reg[2 : 0] oO0;
  reg[(g_AXI_DWIDTH - 1) : 0] iO0;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          iO0 <= { g_AXI_DWIDTH { 1'b0 } };
        end
      else
        begin
          iO0 <= buff_data_in;
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        oO0 <= lol;
      else
        begin
          case (oO0)
            lol:
              begin
                if (start_unpack)
                  oO0 <= ool;
                else
                  oO0 <= lol;
              end
            ool:
              begin
                oO0 <= iol;
              end
            iol:
              begin
                oO0 <= Oil;
              end
            Oil:
              begin
                if (buff_raddr == locs_to_read)
                  oO0 <= OI0;
                else
                  oO0 <= Iil;
              end
            Iil:
              begin
                oO0 <= Oil;
              end
            OI0:
              begin
                oO0 <= lol;
              end
            default:
              oO0 <= lol;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
          buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          unpacker_compl <= 1'b0;
          data_valid <= 1'b0;
        end
      else
        begin
          case (oO0)
            lol:
              begin
                video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
                buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                unpacker_compl <= 1'b0;
                data_valid <= 1'b0;
              end
            ool:
              begin
              end
            iol:
              begin
                buff_raddr <= buff_raddr + 1'b1;
              end
            Oil:
              begin
                video_data <= iO0[31 : 0];
                data_valid <= 1'b1;
              end
            Iil:
              begin
                video_data <= iO0[63 : 32];
                data_valid <= 1'b1;
                buff_raddr <= buff_raddr + 1'b1;
              end
            OI0:
              begin
                video_data <= iO0[63 : 32];
                data_valid <= 1'b1;
                unpacker_compl <= 1'b1;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
