/*****************************************************************************************************************************
--
--    File Name    : bus_cdc_synchornizer.v

--    Description  : This module implements CDC synchronizer for buses
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ns/1ps
module bus_cdc_synchornizer #(parameter g_BUS_WIDTH = 32)
                            (input wire DEST_CLOCK_I,
                             input wire[(g_BUS_WIDTH - 1) : 0] input_bus_i,
                             output reg[(g_BUS_WIDTH - 1) : 0] output_bus_o);
  wire[(g_BUS_WIDTH - 1) : 0] Ii;
  reg[(g_BUS_WIDTH - 1) : 0] li,
                             oi;
  integer oo;
  assign Ii = (input_bus_i >> 1) ^ input_bus_i;
  always
    @(posedge DEST_CLOCK_I)
    begin
      li <= Ii;
      oi <= li;
    end
  always
    @(oi)
    begin
      for (oo = 0; oo < g_BUS_WIDTH; oo = oo + 1)
        begin
          output_bus_o[oo] = ^(oi >> oo);
        end
    end
endmodule
