/*****************************************************************************************************************************
--
--    File Name    : cdc_fifo_logic.v

--    Description  : This module passes the valid input data and control signals from write clock domian to read clock domain.
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module cdc_fifo_logic #(parameter g_DATAWIDTH = 8,
                        parameter g_BUFF_DEPTH = 1280)
                      (input wire CAM_CLOCK_I,
                       input wire READ_CLOCK_I,
                       input wire RESET_n_I,
                       input wire[(g_DATAWIDTH - 1) : 0] pixel_in_i,
                       input wire pixel_in_vld_i,
                       input wire frame_done_i,
                       input wire frame_start_i,
                       input wire[15 : 0] word_count_i,
                       input wire line_start_i,
                       output reg[(g_DATAWIDTH - 1) : 0] pixel_out_o,
                       output reg pixel_vld_o,
                       output wire line_start_o,
                       output wire frame_start_o,
                       output reg line_done_o,
                       output wire frame_done_o);
  reg io,
      Oi;
  reg[(g_DATAWIDTH - 1) : 0] Ii;
  reg[15 : 0] li;
  reg[15 : 0] oi;
  reg[15 : 0] ii;
  wire[15 : 0] OOI;
  reg IOI;
  reg lOI;
  wire[(g_DATAWIDTH - 1) : 0] oOI;
  reg iOI;
  reg OII;
  reg III,
      lII,
      oII;
  wire iII;
  reg OlI,
      IlI,
      llI;
  wire olI;
  reg ilI,
      O0I,
      I0I;
  wire l0I;
  wire[15 : 0] o0I;
  assign iII = line_start_i ? ~OlI : OlI;
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          OlI <= 1'd0;
        end
      else
        begin
          OlI <= iII;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          IlI <= 1'd0;
        end
      else
        begin
          IlI <= OlI;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          llI <= 1'd0;
        end
      else
        begin
          llI <= IlI;
        end
    end
  assign line_start_o = llI ^ IlI;
  assign olI = frame_done_i ? ~ilI : ilI;
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          ilI <= 1'd0;
        end
      else
        begin
          ilI <= olI;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          O0I <= 1'd0;
        end
      else
        begin
          O0I <= ilI;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          I0I <= 1'd0;
        end
      else
        begin
          I0I <= O0I;
        end
    end
  assign frame_done_o = I0I ^ O0I;
  assign l0I = frame_start_i ? ~III : III;
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          III <= 1'd0;
        end
      else
        begin
          III <= l0I;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          lII <= 1'd0;
        end
      else
        begin
          lII <= III;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          oII <= 1'd0;
        end
      else
        begin
          oII <= lII;
        end
    end
  assign frame_start_o = oII ^ lII;
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          iOI <= 1'b0;
          OII <= 1'b0;
        end
      else
        begin
          iOI <= frame_start_i;
          OII <= line_start_i;
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          Ii <= 8'd0;
        end
      else
        begin
          if (OII == 1'b1 || iOI == 1'b1)
            begin
              Ii <= 8'd0;
            end
          else
            begin
              if (pixel_in_vld_i == 1'b1)
                begin
                  Ii <= pixel_in_i;
                end
            end
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          li <= 16'hFFFF;
        end
      else
        begin
          if (OII == 1'b1 || iOI == 1'b1)
            begin
              li <= 16'd0;
            end
          else
            if (li < (o0I) && pixel_in_vld_i == 1'b1)
              begin
                li <= li + 1'b1;
              end
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          IOI <= 1'b0;
        end
      else
        begin
          if (li < (o0I) && pixel_in_vld_i == 1'b1)
            begin
              IOI <= 1'b1;
            end
          else
            begin
              IOI <= 1'b0;
            end
        end
    end
  always
    @(posedge (CAM_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          oi <= 16'hFFFF;
        end
      else
        begin
          oi <= li;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          lOI <= 1'b0;
        end
      else
        begin
          if (ii <= ((o0I) - 1))
            begin
              lOI <= 1'b1;
            end
          else
            begin
              lOI <= 1'b0;
            end
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          io <= 1'b0;
        end
      else
        begin
          if (ii == (o0I - 1))
            begin
              io <= 1'b1;
            end
          else
            begin
              io <= 1'b0;
            end
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          Oi <= 1'd0;
          line_done_o <= 1'b0;
        end
      else
        begin
          line_done_o <= Oi;
          Oi <= io;
        end
    end
  always
    @(posedge (READ_CLOCK_I) or
      negedge (RESET_n_I))
    begin
      if (!RESET_n_I)
        begin
          ii <= 16'hFFFF;
        end
      else
        begin
          if (ii <= (o0I - 1))
            begin
              ii <= ii + 1'd1;
            end
          else
            begin
              if (OOI == (o0I - 16'd100))
                begin
                  ii <= 16'd0;
                end
            end
        end
    end
  ram2port_cam #(.g_BUFF_AWIDTH(16), .g_DWIDTH(g_DATAWIDTH), .BUFF_DEPTH(g_BUFF_DEPTH)) i0I(.wr_data_i(Ii),
                                                                                            .rd_addr(ii),
                                                                                            .wr_addr(oi),
                                                                                            .we(IOI),
                                                                                            .wclock(CAM_CLOCK_I),
                                                                                            .rclock(READ_CLOCK_I),
                                                                                            .rd_data_o(oOI));
  bus_cdc_synchronizer #(.g_BUS_WIDTH(16)) O1I(.input_bus_i(oi),
                                               .output_bus_o(OOI),
                                               .DEST_CLOCK_I(READ_CLOCK_I));
  bus_cdc_synchronizer #(.g_BUS_WIDTH(16)) I1I(.input_bus_i(word_count_i),
                                               .output_bus_o(o0I),
                                               .DEST_CLOCK_I(READ_CLOCK_I));
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          pixel_out_o <= { g_DATAWIDTH { 1'b0 } };
          pixel_vld_o <= 1'b0;
        end
      else
        begin
          pixel_out_o <= oOI;
          pixel_vld_o <= lOI;
        end
    end
endmodule
