/*****************************************************************************************************************************
--
--    File Name    : mipi_csi2_rxdecoder.v 

--    Description  : This module implements High Speed Data decoding of MIPI CSI-2 interface.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module mipi_csi2_rxdecoder #(parameter g_DATAWIDTH = 8,
                             parameter g_INPUT_DATA_INVERT = 0,
                             parameter g_LANE_WIDTH = 4,
                             parameter g_BUFF_DEPTH = 1280)
                           (input CAM_CLOCK_I,
                            input READ_CLOCK_I,
                            input RESET_n_I,
                            input wire[g_DATAWIDTH - 1 : 0] ChannelA_i,
                            input wire[g_DATAWIDTH - 1 : 0] ChannelB_i,
                            input wire[g_DATAWIDTH - 1 : 0] ChannelC_i,
                            input wire[g_DATAWIDTH - 1 : 0] ChannelD_i,
                            output wire[g_DATAWIDTH - 1 : 0] data_out_o,
                            output reg[15 : 0] word_count_o,
                            output wire pixel_valid_o,
                            output reg frame_start_o,
                            output reg frame_end_o,
                            output reg line_start_o,
                            output reg line_end_o,
                            output reg[15 : 0] frame_number_o);
  parameter l1o = 16;
  parameter o1o = 16'hb800;
  parameter i1o = 16'hb801;
  parameter Ooo = 16'hb82a;
  parameter Ioo = 0,
            loo = 1,
            ooo = 2,
            ioo = 3,
            Oio = 4,
            Iio = 5,
            lio = 6,
            oio = 7,
            iio = 8,
            OOi = 9,
            IOi = 10,
            lOi = 11,
            oOi = 12,
            iOi = 13,
            OIi = 14,
            IIi = 15;
  parameter lIi = 0,
            oIi = 1,
            iIi = 2,
            Oli = 3,
            Ili = 4,
            lli = 5,
            oli = 6,
            ili = 7;
  parameter O0i = 0,
            I0i = 1,
            l0i = 2,
            o0i = 3,
            i0i = 4,
            O1i = 5;
  reg[7 : 0] I1i,
             l1i;
  reg o1i;
  reg i1i;
  reg Ooi;
  reg Ioi;
  reg loi;
  reg[7 : 0] ooi,
             ioi,
             Oii,
             Iii;
  reg[15 : 0] lii;
  reg[7 : 0] oii;
  reg[7 : 0] iii;
  reg[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] OOOI,
                                          IOOI;
  reg[3 : 0] lOOI;
  reg[l1o - 1 : 0] oOOI;
  wire iOOI;
  wire OIOI;
  wire IIOI;
  wire lIOI;
  wire oIOI;
  wire iIOI;
  wire OlOI;
  wire IlOI;
  wire llOI;
  reg olOI,
      ilOI;
  reg O0OI,
      I0OI;
  assign iOOI = 1'b0;
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ooi <= 8'd0;
        end
      else
        if (!iOOI)
          begin
            if (g_INPUT_DATA_INVERT)
              ooi <= { ChannelA_i[0], ChannelA_i[1], ChannelA_i[2], ChannelA_i[3], ChannelA_i[4], ChannelA_i[5], ChannelA_i[6], ChannelA_i[7] };
            else
              ooi <= ChannelA_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        ioi <= 8'd0;
      else
        if (!iOOI)
          begin
            if (g_INPUT_DATA_INVERT)
              ioi <= { ChannelB_i[0], ChannelB_i[1], ChannelB_i[2], ChannelB_i[3], ChannelB_i[4], ChannelB_i[5], ChannelB_i[6], ChannelB_i[7] };
            else
              ioi <= ChannelB_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Oii <= 8'd0;
        end
      else
        if (!iOOI)
          begin
            if (g_INPUT_DATA_INVERT)
              Oii <= { ChannelC_i[0], ChannelC_i[1], ChannelC_i[2], ChannelC_i[3], ChannelC_i[4], ChannelC_i[5], ChannelC_i[6], ChannelC_i[7] };
            else
              Oii <= ChannelC_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        Iii <= 8'd0;
      else
        if (!iOOI)
          begin
            if (g_INPUT_DATA_INVERT)
              Iii <= { ChannelD_i[0], ChannelD_i[1], ChannelD_i[2], ChannelD_i[3], ChannelD_i[4], ChannelD_i[5], ChannelD_i[6], ChannelD_i[7] };
            else
              Iii <= ChannelD_i;
          end
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OOOI[7 : 0] <= 8'b0;
        end
      else
        if (g_LANE_WIDTH >= 1)
          begin
            OOOI[7 : 0] <= ooi;
          end
        else
          OOOI[7 : 0] <= 8'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OOOI[15 : 8] <= 8'b0;
        end
      else
        if (g_LANE_WIDTH >= 2)
          begin
            OOOI[15 : 8] <= { ioi };
          end
        else
          OOOI[15 : 8] <= 8'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          OOOI[31 : 16] <= 16'b0;
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            OOOI[31 : 16] <= { Iii, Oii };
          end
        else
          OOOI[31 : 16] <= 16'b0;
    end
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          oOOI <= 16'b0;
        end
      else
        if (!iOOI)
          begin
            oOOI <= { oOOI[l1o - 9 : 0], OOOI[7 : 0] };
          end
    end
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          begin
            if (!RESET_n_I)
              begin
                lOOI <= 4'b0;
                o1i <= 1'b0;
                i1i <= 1'b0;
                Ooi <= 1'b0;
                loi <= 1'b0;
                lii <= 16'b0;
                l1i <= 8'd0;
                I1i <= 8'd0;
                frame_number_o <= 16'd0;
                oii <= 8'd0;
                iii <= 8'd0;
                word_count_o <= 16'hFFFF;
                IOOI <= 8'd0;
              end
            else
              if (!iOOI)
                begin
                  case (lOOI)
                    Ioo:
                      begin
                        if (oOOI == o1o)
                          begin
                            o1i <= 1;
                            lOOI <= loo;
                          end
                        else
                          lOOI <= Ioo;
                      end
                    loo:
                      begin
                        lOOI <= ooo;
                        o1i <= 0;
                        I1i <= oOOI[7 : 0];
                      end
                    ooo:
                      begin
                        lOOI <= ioo;
                        o1i <= 0;
                        l1i <= oOOI[7 : 0];
                      end
                    ioo:
                      begin
                        lOOI <= Oio;
                        o1i <= 0;
                        frame_number_o <= { l1i, I1i };
                      end
                    Oio:
                      begin
                        if (oOOI == i1o)
                          begin
                            i1i <= 1;
                            lOOI <= oOi;
                            Ooi <= 0;
                            o1i <= 0;
                          end
                        else
                          if (oOOI == Ooo)
                            begin
                              loi <= 0;
                              lOOI <= Iio;
                              Ooi <= 1'b1;
                            end
                          else
                            begin
                              loi <= 0;
                              lOOI <= Oio;
                              Ooi <= 0;
                            end
                      end
                    Iio:
                      begin
                        Ooi <= 0;
                        oii <= oOOI[7 : 0];
                        lOOI <= lio;
                      end
                    lio:
                      begin
                        Ooi <= 0;
                        iii <= oOOI[7 : 0];
                        lOOI <= oio;
                      end
                    oio:
                      begin
                        word_count_o <= { iii, oii };
                        lOOI <= iio;
                      end
                    iio:
                      begin
                        if (lii < word_count_o)
                          begin
                            lii <= lii + 1;
                            loi <= 1;
                            IOOI <= oOOI[7 : 0];
                            lOOI <= iio;
                          end
                        else
                          begin
                            lii <= 0;
                            loi <= 0;
                            IOOI <= 0;
                            lOOI <= OOi;
                          end
                      end
                    OOi:
                      begin
                        lOOI <= IOi;
                      end
                    IOi:
                      begin
                        lOOI <= lOi;
                      end
                    lOi:
                      begin
                        lOOI <= Oio;
                      end
                    oOi:
                      begin
                        i1i <= 0;
                        lOOI <= iOi;
                        o1i <= 1'b0;
                      end
                    iOi:
                      begin
                        lOOI <= OIi;
                      end
                    OIi:
                      begin
                        lOOI <= IIi;
                      end
                    IIi:
                      begin
                        lOOI <= Ioo;
                      end
                    default:
                      lOOI <= Ioo;
                  endcase
                end
          end
      end
    else
      if (g_LANE_WIDTH == 2)
        begin
          always
            @(posedge CAM_CLOCK_I or
              negedge RESET_n_I)
            begin
              if (!RESET_n_I)
                begin
                  lOOI <= lIi;
                  o1i <= 1'b0;
                  i1i <= 1'b0;
                  Ooi <= 1'b0;
                  Ioi <= 1'b0;
                  loi <= 1'b0;
                  lii <= 16'b0;
                  l1i <= 8'd0;
                  I1i <= 8'd0;
                  frame_number_o <= 16'd0;
                  oii <= 8'd0;
                  iii <= 8'd0;
                  word_count_o <= 16'hFFFF;
                  IOOI <= 0;
                end
              else
                case (lOOI)
                  lIi:
                    begin
                      i1i <= 0;
                      if (OOOI == 16'hB8B8)
                        lOOI <= oIi;
                      else
                        lOOI <= lIi;
                    end
                  oIi:
                    if (OOOI[5 : 0] == 6'h00)
                      begin
                        o1i <= 1'b1;
                        I1i <= OOOI[15 : 8];
                        lOOI <= iIi;
                      end
                    else
                      lOOI <= lIi;
                  iIi:
                    begin
                      lOOI <= Oli;
                      o1i <= 1'b0;
                      frame_number_o <= { OOOI[7 : 0], I1i };
                    end
                  Oli:
                    begin
                      Ioi <= 1'b0;
                      o1i <= 0;
                      if (OOOI == 16'hB8B8)
                        lOOI <= Ili;
                      else
                        lOOI <= Oli;
                      lii <= 0;
                    end
                  Ili:
                    if (OOOI[5 : 0] == 6'h01)
                      begin
                        lOOI <= lIi;
                        i1i <= 1'b1;
                      end
                    else
                      begin
                        oii <= OOOI[15 : 8];
                        lOOI <= lli;
                        Ooi <= 1'b1;
                      end
                  lli:
                    begin
                      word_count_o <= { OOOI[7 : 0], oii };
                      lOOI <= oli;
                      Ooi <= 1'b0;
                    end
                  oli:
                    begin
                      Ooi <= 1'b0;
                      lii <= lii + 2;
                      loi <= 1;
                      IOOI <= OOOI;
                      if (lii < (word_count_o))
                        lOOI <= oli;
                      else
                        lOOI <= ili;
                    end
                  ili:
                    begin
                      lOOI <= Oli;
                      loi <= 0;
                      Ioi <= 1'b1;
                      IOOI <= 0;
                    end
                  default:
                    begin
                      lOOI <= lIi;
                      o1i <= 1'b0;
                      i1i <= 1'b0;
                      Ooi <= 1'b0;
                      Ioi <= 1'b0;
                      loi <= 1'b0;
                      lii <= 16'b0;
                      l1i <= 8'd0;
                      I1i <= 8'd0;
                      frame_number_o <= 16'd0;
                      oii <= 8'd0;
                      iii <= 8'd0;
                      word_count_o <= 16'hFFFF;
                      IOOI <= 0;
                    end
                endcase
            end
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            always
              @(posedge CAM_CLOCK_I or
                negedge RESET_n_I)
              begin
                if (!RESET_n_I)
                  begin
                    lOOI <= O0i;
                    o1i <= 1'b0;
                    i1i <= 1'b0;
                    Ooi <= 1'b0;
                    Ooi <= 1'b0;
                    loi <= 1'b0;
                    lii <= 16'b0;
                    l1i <= 8'd0;
                    I1i <= 8'd0;
                    frame_number_o <= 16'd0;
                    oii <= 8'd0;
                    iii <= 8'd0;
                    word_count_o <= 16'hFFFF;
                    IOOI <= 0;
                  end
                else
                  case (lOOI)
                    O0i:
                      begin
                        i1i <= 0;
                        if (OOOI == 32'hB8B8B8B8)
                          lOOI <= I0i;
                        else
                          lOOI <= O0i;
                      end
                    I0i:
                      if (OOOI[5 : 0] == 6'h00)
                        begin
                          o1i <= 1'b1;
                          frame_number_o <= OOOI[23 : 8];
                          lOOI <= 4'h2;
                        end
                      else
                        lOOI <= O0i;
                    l0i:
                      begin
                        Ioi <= 1'b0;
                        o1i <= 0;
                        if (OOOI == 32'hB8B8B8B8)
                          lOOI <= o0i;
                        else
                          lOOI <= l0i;
                        lii <= 0;
                      end
                    o0i:
                      if (OOOI[5 : 0] == 6'h01)
                        begin
                          lOOI <= O0i;
                          i1i <= 1'b1;
                        end
                      else
                        begin
                          word_count_o <= OOOI[23 : 8];
                          lOOI <= i0i;
                          Ooi <= 1'b1;
                        end
                    i0i:
                      begin
                        Ooi <= 1'b0;
                        lii <= lii + 4;
                        loi <= 1;
                        IOOI <= OOOI;
                        if (lii < (word_count_o))
                          lOOI <= i0i;
                        else
                          lOOI <= O1i;
                      end
                    O1i:
                      begin
                        lOOI <= l0i;
                        loi <= 0;
                        Ioi <= 1'b1;
                        IOOI <= 0;
                      end
                    default:
                      begin
                        lOOI <= O0i;
                        o1i <= 1'b0;
                        i1i <= 1'b0;
                        Ooi <= 1'b0;
                        Ioi <= 1'b0;
                        loi <= 1'b0;
                        lii <= 16'b0;
                        l1i <= 8'd0;
                        I1i <= 8'd0;
                        frame_number_o <= 16'd0;
                        oii <= 8'd0;
                        iii <= 8'd0;
                        word_count_o <= 16'hFFFF;
                        IOOI <= 0;
                      end
                  endcase
              end
          end
  endgenerate
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                frame_start_o <= 1'd0;
              end
            else
              begin
                frame_start_o <= oIOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                line_end_o <= 1'd0;
              end
            else
              begin
                line_end_o <= OIOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                frame_end_o <= 1'd0;
              end
            else
              begin
                frame_end_o <= lIOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                line_start_o <= 1'd0;
              end
            else
              begin
                line_start_o <= IIOI;
              end
          end
        cdc_fifo_logic #(.g_DATAWIDTH(g_DATAWIDTH), .g_BUFF_DEPTH(g_BUFF_DEPTH)) l0OI(.CAM_CLOCK_I(CAM_CLOCK_I),
                                                                                      .READ_CLOCK_I(READ_CLOCK_I),
                                                                                      .RESET_n_I(RESET_n_I),
                                                                                      .pixel_in_i(IOOI),
                                                                                      .pixel_in_vld_i(loi),
                                                                                      .frame_done_i(i1i),
                                                                                      .frame_start_i(o1i),
                                                                                      .word_count_i(word_count_o),
                                                                                      .line_start_i(Ooi),
                                                                                      .pixel_out_o(data_out_o),
                                                                                      .pixel_vld_o(pixel_valid_o),
                                                                                      .line_start_o(IIOI),
                                                                                      .frame_start_o(oIOI),
                                                                                      .line_done_o(OIOI),
                                                                                      .frame_done_o(lIOI));
      end
    else
      begin
        byte2pixel_conversion #(.g_DATAWIDTH(g_DATAWIDTH), .g_LANE_WIDTH(g_LANE_WIDTH)) o0OI(.CAM_CLOCK_I(CAM_CLOCK_I),
                                                                                             .READ_CLOCK_I(READ_CLOCK_I),
                                                                                             .RESET_n_I(RESET_n_I),
                                                                                             .byte_data_i(IOOI),
                                                                                             .byte_en_i(loi),
                                                                                             .pixel_data_o(data_out_o),
                                                                                             .line_valid_o(pixel_valid_o),
                                                                                             .line_start_o(iIOI),
                                                                                             .line_end_o(OlOI));
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              olOI <= 0;
              ilOI <= 0;
            end
          else
            begin
              olOI <= o1i;
              ilOI <= olOI;
            end
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              O0OI <= 0;
              I0OI <= 0;
            end
          else
            begin
              O0OI <= i1i;
              I0OI <= O0OI;
            end
        assign IlOI = olOI & ~ilOI;
        assign llOI = O0OI & ~I0OI;
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                frame_start_o <= 1'd0;
              end
            else
              begin
                frame_start_o <= IlOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                frame_end_o <= 1'd0;
              end
            else
              begin
                frame_end_o <= llOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                line_start_o <= 1'd0;
              end
            else
              begin
                line_start_o <= iIOI;
              end
          end
        always
          @(posedge (READ_CLOCK_I) or
            negedge (RESET_n_I))
          begin
            if (!RESET_n_I)
              begin
                line_end_o <= 1'd0;
              end
            else
              begin
                line_end_o <= OlOI;
              end
          end
      end
  endgenerate
endmodule
